import importlib
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import RandomizedSearchCV
from sklearn.neighbors import KNeighborsClassifier
from sklearn.svm import SVC
from os import path,stat
import time
import pickle
import numpy as np
from importlib import import_module
import tensorflow as tf
import pandas as pd
import os
import sys
sys.path.append(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
config = importlib.import_module('config')
data_utils = importlib.import_module('data_utils')
other_utils = importlib.import_module('other_utils')

class Utils:
    def __init__(self):
        RF_parameters = {
            'n_estimators': range(20, 800, 10),
            'criterion': ['entropy', 'gini'],
            'max_depth': range(5, 25, 1),
            'max_features': ['auto', 'sqrt', 'log2'],
            'min_samples_split': range(2, 5, 1),
        }

        KNN_parameters = {
            'n_neighbors': range(1, 10, 1),
            'weights': ['uniform', 'distance'],
            'algorithm': ['auto', 'ball_tree', 'kd_tree', 'brute']
        }

        SVC_parameters = {
            'C': np.arange(1, 8, 0.2),
            'kernel': ['linear', 'poly', 'rbf', 'sigmoid'],
            'gamma': ['scale', 'auto'],
            'degree': range(1, 11, 1)
        }

        self.param_dict = {'RF': RF_parameters,
                      'KNN': KNN_parameters,
                      'SVC': SVC_parameters}
        self.model_name_list = ['RF','KNN','SVC']

        self.cla_dict ={'RF':RandomForestClassifier(),
                           'KNN':KNeighborsClassifier(),
                           'SVC':SVC()}

    def __get_model_best(self,model_name,task_path,force_refresh):

            save_path = path.join(task_path, '{}_ini.pkl'.format(model_name))

            print('最优化模型{},{}......'.format(model_name,path.basename(task_path)))
            start = time.time()

            if path.exists(save_path) and not force_refresh:
                if stat(save_path).st_size > 0:
                    print('最优化模型已存在，跳过......')
                    return None

            x,y = data_utils.utils.get_xy(task_path)


            random_search = RandomizedSearchCV(self.cla_dict[model_name],
                                               param_distributions=self.param_dict[model_name],
                                               n_iter=config.config.random_search_n_iter,
                                               cv=config.config.cross_val_num,
                                               scoring='accuracy',
                                               random_state = config.config.random_num
                                               # n_jobs= -1
                                               )

            random_search.fit(x,y)

            best_params = random_search.cv_results_['params'][random_search.best_index_]

            with open(save_path,'wb') as f:
                pickle.dump(best_params,f)

            end = time.time()
            print('优化完成，耗时:{}'.format(end-start))

    def get_all_model_best(self,task_path,force_refresh=False):
        for model_name in self.model_name_list:
            self.__get_model_best(model_name,task_path,force_refresh)

    def get_model(self,model_name,task_path,initial_weigh=None):

        if model_name not in ['SANN','ANN']:
            with open(path.join(task_path, model_name + '_ini.pkl'),'rb') as f:
                ini_dict = pickle.load(f)
        else:
            x, y = data_utils.utils.get_xy_onehot(task_path)

        if model_name == 'RF':
            model = RandomForestClassifier(n_estimators=ini_dict['n_estimators'],
                                           criterion=ini_dict['criterion'],
                                           max_depth=ini_dict['max_depth'],
                                           max_features=ini_dict['max_features'],
                                           min_samples_split=ini_dict['min_samples_split'],
                                           random_state = config.config.random_num
                                           )

        elif model_name == 'KNN':
            model = KNeighborsClassifier(n_neighbors=ini_dict['n_neighbors'],
                                         weights=ini_dict['weights'],
                                         algorithm=ini_dict['algorithm']
                                         )

        elif model_name == 'SVC':
            model =SVC(
                C=ini_dict['C'],
                kernel=ini_dict['kernel'],
                gamma=ini_dict['gamma'],
                degree=ini_dict['degree'],
                random_state = config.config.random_num)


        elif model_name in ['SANN','ANN']:

            model_file = import_module('model.{}'.format(model_name))
            model = model_file.Model(trainable=True, initial_weigh=initial_weigh)



        return model



utils = Utils()

if __name__ == '__main__':
    pass


import os
import configparser




class Config:
    def __init__(self):

        cg = configparser.ConfigParser()
        cg.read(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))),'initial.ini'), encoding='utf-8')


        self.input_path = cg.get('path','input_path')


        task_name_list_str = cg.get('task', 'task_name_list').strip(',')
        self.task_name_list = task_name_list_str.split(',')


        self.task_path_list = [os.path.join(self.input_path,task_name) for task_name in self.task_name_list]
        self.model_path = 'model.simple_net'
        self.cross_val_num = 5
        self.epochs_num = 1000
        self.drop_rate = 0.5
        self.random_search_n_iter = 100
        self.random_search_n_jobs = -1
        self.batch_size = 300
        self.model_list = ['SANN','ANN','KNN','SVC','RF']
        self.metrics_list = ['accuracy','precision','recall','f1']
        self.random_num = 100
        self.legend_loc = 3
        self.patience_num = 50

config = Config()

if __name__ == '__main__':
    pass


import sys
import os

sys.path.append(os.path.dirname(__file__))

# 导入sann模块的所有包
import configparser




# 定义SANN类
class Sann():
    def __init__(self,input_path,task_name_list):

        cg = configparser.ConfigParser()
        cg['path'] = {'input_path' : input_path,

                             }

        task_name_list_str = ''
        for task_name in task_name_list:
            task_name_list_str += ','
            task_name_list_str += task_name

        cg['task'] = {'task_name_list': task_name_list_str,
                      'cur_task_path': 'None'
                      }

        with open(os.path.join(os.path.join(os.path.dirname(os.path.abspath(__file__)),'sann'),'initial.ini'), 'w',encoding='utf-8') as f:
            cg.write(f)

        from jhsp.sann.bin import config
        from jhsp.sann.bin import other_utils
        from jhsp.sann.bin import data_utils
        from jhsp.sann.bin import model_utils

        self.config = config.config
        self.other_utils = other_utils.utils
        self.data_utils = data_utils.utils
        self.model_utils = model_utils.utils

# 实例化sann
# 在调用时实例化，因为要传入一个工作目录

sann = Sann('../1/',['不孕症'])
# sann.data_utils.get_weigh_corr(task_path=sann.config.task_path_list[0],force_refresh=True)
# sann.model_utils.get_all_model_best(task_path=sann.config.task_path_list[0],force_refresh=True)
sann.other_utils.get_models2ills_scores(sann.config.task_path_list[0],sann.config.cross_val_num,re_get_score=False,force_refresh=False)
sann.other_utils.plot_epochs_scores(500,task_path=sann.config.task_path_list[0],force_refresh=False)
sann.other_utils.plot_model2ill_score('f1','不孕症',force_refresh=True)












